﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Media;

namespace Vis2NodeTrix
{
    /// <summary>
    /// Handler for animation events
    /// </summary>
    /// <param name="sender">Object starts the animation</param>
    /// <param name="e">Arguments of the animation</param>
    public delegate void AnimationEventHandler(object sender, AnimationEventArgs e);

    /// <summary>
    /// Defines the type of Animation
    /// </summary>
    public enum AnimationType
    {
        /// <summary>
        /// Animation which makes nodes move together to a matrix
        /// </summary>
        TRANSLATE_TO_MATRIX, 
        /// <summary>
        /// Animation which explodes a matrix to nodes
        /// </summary>
        TRANSLATE_FROM_MATRIX, 
        /// <summary>
        /// Animation which rotates
        /// </summary>
        ROTATE, 
        /// <summary>
        /// Animation which scales
        /// </summary>
        SCALE, 
        /// <summary>
        /// No animation
        /// </summary>
        NONE
    }

    /// <summary>
    /// Class for AnimationEvent arguments
    /// </summary>
    public class AnimationEventArgs : EventArgs
    {
        /// <summary>
        /// Type of the animation
        /// </summary>
        public AnimationType Type { get; internal set; }
        /// <summary>
        /// Array of values which are important for the animation (size depends on the animation type)
        /// </summary>
        public double[] Values { get; internal set; }
        /// <summary>
        /// Additional data for the animation behavior
        /// </summary>
        public object Data { get; internal set; }

        /// <summary>
        /// Creates an AnimationEventArgs object
        /// </summary>
        /// <param name="type">Type of the animation</param>
        /// <param name="values">Important animation values</param>
        /// <param name="data">Additional data for the animation behavior</param>
        public AnimationEventArgs(AnimationType type, double[] values, object data)
        {
            this.Type = type;
            this.Values = values;
            this.Data = data;
        }
    }
}
