﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GraphVizNet;
using System.ComponentModel;
using System.Windows;

namespace Vis2NodeTrix
{
    /// <summary>
    /// Baseclass for all Nodes used in this program
    /// </summary>
    public abstract class BaseNode : GraphVizNet.VizNode, INotifyPropertyChanged
    {
        /// <summary>
        /// Returns all connections containing this node
        /// </summary>
        public abstract List<VizEdge> Connections { get; }

        /// <summary>
        /// Location of the Node on the screen
        /// </summary>
        public new Point Pos { get { return base.Pos; } set { base.Pos = value; OnPropertyChanged("Pos"); } }

        /// <summary>
        /// Location of the Node on the screen, the PorpertyChanged Event is not thrown
        /// </summary>
        public Point PosNoEvent { get { return base.Pos; } set { base.Pos = value; } }

        private bool isAnimating;

        /// <summary>
        /// Gets or sets a value defining if a node is currently animated
        /// </summary>
        public bool IsAnimating { get { return isAnimating; } set { isAnimating = value; OnPropertyChanged("IsAnimating"); } }

        /// <summary>
        /// Occurs when the Node is animated
        /// </summary>
        public event AnimationEventHandler AnimationEvent;

        /// <summary>
        /// Throws the AnimationEvent
        /// </summary>
        /// <param name="type">The type of the animation</param>
        /// <param name="values">A array with values defining the animation</param>
        /// <param name="data">The data for the animation</param>
        public void OnAnimation(AnimationType type, double[] values, object data)
        {
            if (!(this.AnimationEvent == null))
            {
                AnimationEvent(this, new AnimationEventArgs(type, values, data));
            }
        }

        /// <summary>
        /// Throws the PropertyChangedEvent
        /// </summary>
        /// <param name="_property">Name of the property</param>
        public void OnPropertyChanged(string _property)
        {
            if (this.PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(_property));
        }

        /// <summary>
        /// Occurs when the value of a Property is set
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Returns the next valid point on the nodes boundingbox in direction of the edge
        /// </summary>
        /// <param name="_edge">Edge defining the direction</param>
        /// <returns>The nearest point on the boundingbox</returns>
        public abstract Point PointOnBox(VizEdge _edge);
    }
}
