﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using System.Windows;
using GraphVizNet;
using System.Windows.Media;
using System.ComponentModel;

namespace Vis2NodeTrix
{
    class EdgePresenter : Control, INotifyPropertyChanged
    {
        private Vector currentLength;

        static EdgePresenter()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(EdgePresenter),
                new FrameworkPropertyMetadata(typeof(EdgePresenter)));
        }

        public EdgePresenter(VizEdge edge)
        {
            Edge = edge;
            (edge.Head as SimpleNode).PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(EdgePresenter_PropertyChanged);
            (edge.Tail as SimpleNode).PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(EdgePresenter_PropertyChanged);

            if ((edge.Head as SimpleNode).IsGrouped)
                (edge.Head as SimpleNode).ParentNode.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(EdgePresenter_PropertyChanged);
            if ((edge.Tail as SimpleNode).IsGrouped)
                (edge.Tail as SimpleNode).ParentNode.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(EdgePresenter_PropertyChanged);

            currentLength = HeadNode.Pos - TailNode.Pos;

            if (edge.Points.Count == 0) //Wenn noch nie gelayouted, dann 4 punkte einfügen
            {
                for (double i = 0; i < 4; i++)
                {
                    edge.Points.Add(Edge.Tail.Pos + currentLength * i / 3.0);
                }
            }

            DoBoundingBoxCorrection();
        }

        public void RemoveMyHandler()
        {
            (Edge.Head as SimpleNode).PropertyChanged -= new System.ComponentModel.PropertyChangedEventHandler(EdgePresenter_PropertyChanged);
            (Edge.Tail as SimpleNode).PropertyChanged -= new System.ComponentModel.PropertyChangedEventHandler(EdgePresenter_PropertyChanged);

            if ((Edge.Head as SimpleNode).IsGrouped)
                (Edge.Head as SimpleNode).ParentNode.PropertyChanged -= new System.ComponentModel.PropertyChangedEventHandler(EdgePresenter_PropertyChanged);
            if ((Edge.Tail as SimpleNode).IsGrouped)
                (Edge.Tail as SimpleNode).ParentNode.PropertyChanged -= new System.ComponentModel.PropertyChangedEventHandler(EdgePresenter_PropertyChanged);
        }

        public void UpdateCurrentLength()
        {
            currentLength = HeadNode.Pos - TailNode.Pos;
        }

        public void UpdateLine(object sender)
        {
            Vector newLen = HeadNode.Pos - TailNode.Pos;

            if (Math.Abs(newLen.X) < (HeadNode.Size.Width + TailNode.Size.Width) / 2.0 &&
                Math.Abs(newLen.Y) < (HeadNode.Size.Height + TailNode.Size.Height) / 2.0)
            {
                this.Visibility = System.Windows.Visibility.Hidden;
            }
            else
            {
                this.Visibility = System.Windows.Visibility.Visible;
            }

            if (newLen != currentLength)
            {
                double scaleX = newLen.X / currentLength.X;
                if (Math.Abs(currentLength.X) <= 1.0)
                    scaleX = newLen.X;
                if (Math.Abs(scaleX) <= 0.01)
                    scaleX = 0.01 * Math.Sign(scaleX);

                double scaleY = newLen.Y / currentLength.Y;
                if (Math.Abs(currentLength.Y) <= 1.0)
                    scaleY = newLen.Y;
                if (Math.Abs(scaleY) <= 0.01)
                    scaleY = 0.01 * Math.Sign(scaleY);



                Point refPoint = HeadNode.Pos;
                if (sender == HeadNode)
                    refPoint = TailNode.Pos;

                for (int i = 0; i < Edge.Points.Count; i++)
                {
                    Vector v = (Edge.Points[i] - refPoint);

                    if (Math.Abs(v.X) <= 0.000001 && !Double.IsNaN(scaleX))
                        v.X = ((double)i / (double)Edge.Points.Count) * scaleX;
                    else if (!Double.IsNaN(scaleX))
                        v.X *= scaleX;

                    if (Math.Abs(v.Y) <= 0.000001 && !Double.IsNaN(scaleY))
                        v.Y = ((double)i / (double)Edge.Points.Count) * scaleY;
                    else if (!Double.IsNaN(scaleY))
                        v.Y *= scaleY;

                    v += (Vector)refPoint;

                    Edge.Points[i] = new Point(v.X, v.Y);
                }

                currentLength = newLen;
            }

            DoBoundingBoxCorrection();
            MakeGeometry();
        }

        private BaseNode HeadNode { get { if ((Edge.Head as SimpleNode).IsGrouped) { return (Edge.Head as SimpleNode).ParentNode; } else { return Edge.Head as BaseNode; } } }

        private BaseNode TailNode { get { if ((Edge.Tail as SimpleNode).IsGrouped) { return (Edge.Tail as SimpleNode).ParentNode; } else { return Edge.Tail as BaseNode; } } }

        /// <summary>
        /// Moves the start and the endpoint outside of the nodes bounding box
        /// </summary>
        public void DoBoundingBoxCorrection()
        {
            if (Edge.Points.Count >= 2)
            {
                Edge.Points[0] = HeadNode.PointOnBox(Edge);
                Edge.Points[Edge.Points.Count - 1] = TailNode.PointOnBox(Edge);

                for (int i = 1; i < Edge.Points.Count - 1; i++)
                {
                    if (Math.Abs(TailNode.Pos.X - Edge.Points[i].X) < TailNode.Size.Width / 2.0 &&
                        Math.Abs(TailNode.Pos.Y - Edge.Points[i].Y) < TailNode.Size.Height / 2.0)
                    {
                        Edge.Points.RemoveAt(i);
                        i--;
                    }
                }

                for (int i = 1; i < Edge.Points.Count - 1; i++)
                {
                    if (Math.Abs(HeadNode.Pos.X - Edge.Points[i].X) < HeadNode.Size.Width / 2.0 &&
                        Math.Abs(HeadNode.Pos.Y - Edge.Points[i].Y) < HeadNode.Size.Height / 2.0)
                    {
                        Edge.Points.RemoveAt(i);
                        i--;
                    }
                }
            }
        }

        void EdgePresenter_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Pos")
            {
                if (sender == HeadNode || sender == TailNode)
                    UpdateLine(sender);
            }
            else if (e.PropertyName == "BeforeParentNode")
            {
                SimpleNode sNode = sender as SimpleNode;
                if (sNode.IsGrouped)
                    sNode.ParentNode.PropertyChanged -= new System.ComponentModel.PropertyChangedEventHandler(EdgePresenter_PropertyChanged);
            }
            else if (e.PropertyName == "ParentNode")
            {
                SimpleNode sNode = sender as SimpleNode;
                if (sNode.IsGrouped)
                    sNode.ParentNode.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(EdgePresenter_PropertyChanged);

                UpdateLine(sender);
            }
            else if (e.PropertyName == "IsAnimating")
            {
                bool visible = !(Edge.Head as SimpleNode).IsAnimating && !(Edge.Tail as SimpleNode).IsAnimating;
                if (visible)
                    this.Visibility = System.Windows.Visibility.Visible;
                else
                    this.Visibility = System.Windows.Visibility.Hidden;
            }
        }

        public static readonly DependencyProperty EdgeProperty = DependencyProperty.Register("Edge", typeof(VizEdge), typeof(EdgePresenter), new UIPropertyMetadata());
        public VizEdge Edge
        {
            get { return (VizEdge)GetValue(EdgeProperty); }
            set { SetValue(EdgeProperty, value); }
        }

        public static readonly DependencyProperty EdgeGeometryProperty = DependencyProperty.Register("EdgeGeometry", typeof(StreamGeometry), typeof(EdgePresenter), new UIPropertyMetadata());
        public StreamGeometry EdgeGeometry
        {
            get { return (StreamGeometry)GetValue(EdgeGeometryProperty); }
            set { SetValue(EdgeGeometryProperty, value); }
        }  

        internal void MakeGeometry()
        {
            if (Edge.Points.Count > 0)
            {
                StreamGeometry g = new StreamGeometry();
                StreamGeometryContext c = g.Open();

                c.BeginFigure(Edge.Points[0], false, false);
                List<Point> r = new List<Point>(Edge.Points.Where(p => p != Edge.Points[0]));

                if (r.Count % 3 == 1)
                {
                    r.Add(r[r.Count - 1]);
                    r.Add(r[r.Count - 2]);
                }
                else if (r.Count % 3 == 2)
                {
                    r.Insert(r.Count - 2, r[r.Count - 2]);
                }

                
                c.PolyBezierTo(r.ToList(), true, false);

                c.Close();
                g.Freeze();

                EdgeGeometry = g;
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string _property)
        {
            if (this.PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(_property));
        }
    }
}
