﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Windows.Media;
using GraphVizNet;
using System.Windows.Input;

namespace Vis2NodeTrix
{
    class GroupNodePresenter : BaseNodePresenter
    {
        public Point dragPoint;
        public Window draggedObjWindow;
		public Point rightBtnDown;
        private List<TextBlock> verticalBlocks;
        private List<TextBlock> horizontalBlocks;

        static GroupNodePresenter()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(GroupNodePresenter),
                new FrameworkPropertyMetadata(typeof(GroupNodePresenter)));
        }

        public GroupNodePresenter(GroupNode _node, GraphPresenter _gp) : base(_gp)
        {
            Node = _node;

            verticalBlocks = new List<TextBlock>();
            horizontalBlocks = new List<TextBlock>();

            Node.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(Node_PropertyChanged);
            _node.Nodes.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(Nodes_CollectionChanged);

            Matrix = new Grid();
            Matrix.ShowGridLines = true;
            Matrix.SnapsToDevicePixels = true;
            CreateMatrix();

            this.SnapsToDevicePixels = true;

            this.PreviewMouseMove += new System.Windows.Input.MouseEventHandler(GroupNodePresenter_PreviewMouseMove);
            this.Drop += new DragEventHandler(GroupNodePresenter_Drop);
            this.PreviewMouseRightButtonDown += new System.Windows.Input.MouseButtonEventHandler(GroupNodePresenter_PreviewMouseRightButtonDown);
            this.PreviewMouseRightButtonUp += new System.Windows.Input.MouseButtonEventHandler(GroupNodePresenter_PreviewMouseRightButtonUp);

            initialiseHandler();
        }

        void GroupNodePresenter_PreviewMouseRightButtonUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if (e.GetPosition(this) == rightBtnDown)
            {
                double x, y;

                List<SimpleNode> oldNodes = new List<SimpleNode>();

                while(Node.Nodes.Count > 0)
                {
                    SimpleNode n = Node.Nodes.ElementAt(0);
                    Node.RemoveNode(n);
                    gp.Nodes.Add(n);

                    n.Pos = this.Node.Pos;
                    
                    oldNodes.Add(n);
                }

                gp.Nodes.Remove(this.Node);

                List<Vector> directions = new List<Vector>();
                List<Vector> meanVecToNeighbours = new List<Vector>();

                for (int i = 0; i < oldNodes.Count; i++)
                {
                    //Calculate possible directions
                    x = (10 + 15 * oldNodes.Count) * Math.Sin((i + 1) * 2 * Math.PI / oldNodes.Count);
                    y = (10 + 15 * oldNodes.Count) * Math.Cos((i + 1) * 2 * Math.PI / oldNodes.Count);
                    Vector dir = new Vector(x, y);
                    directions.Add(dir);

                    //Calculate the mean Vector for each node
                    meanVecToNeighbours.Add(getMeanVecToNeighbours(oldNodes.ElementAt(i)));
                }

                for (int i = 0; i < directions.Count; i++)
                {
                    Vector d = directions.ElementAt(i);
                    double[] values = { d.X, d.Y };

                    int index = getBestNodeIndex(d, meanVecToNeighbours);
                    SimpleNode n = oldNodes.ElementAt(index);
                                        
                    n.OnAnimation(AnimationType.TRANSLATE_FROM_MATRIX, values, null);

                    oldNodes.RemoveAt(index);
                    meanVecToNeighbours.RemoveAt(index);
                }
            }
        }

        private int getBestNodeIndex(Vector direction, List<Vector> meanVecToNeighbours)
        {
            double max = Double.MinValue;
            int index = 0;
            direction.Normalize();

            for (int i = 0; i < meanVecToNeighbours.Count; i++)
            {
                double value = Vector.Multiply(direction,meanVecToNeighbours.ElementAt(i));
                if(value > max)
                {
                    max = value;
                    index = i;
                }
            }

            return index;
        }

        private Vector getMeanVecToNeighbours(SimpleNode simpleNode)
        {
            Vector mean = new Vector(0,0);

            foreach (VizEdge e in simpleNode.Connections)
            {
                Point target;
                if (e.Head == simpleNode as VizNode)
                    target = e.Tail.Pos;
                else
                    target = e.Head.Pos;

                Vector edgeVec = new Vector(target.X - simpleNode.Pos.X, target.Y - simpleNode.Pos.Y);
                if (edgeVec.Length != 0)
                {
                    edgeVec.Normalize();
                    mean = mean + edgeVec;
                    mean.Normalize();
                }
            }

            return mean;
        }

        void GroupNodePresenter_PreviewMouseRightButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            rightBtnDown = e.GetPosition(this);
        }

        void RowColumn_PreviewMouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (e.LeftButton == System.Windows.Input.MouseButtonState.Pressed && !Keyboard.IsKeyDown(Key.LeftCtrl))
            {
                DataObject dragData = new DataObject("GroupNodeChild", ((FrameworkElement)sender).Tag);

                GiveFeedbackEventHandler handler = new GiveFeedbackEventHandler(Drag_GiveFeedback);
                this.GiveFeedback += handler;
                QueryContinueDragEventHandler queryHandler = new QueryContinueDragEventHandler(Drag_Query);
                this.QueryContinueDrag += queryHandler;

                draggedObjWindow = creatDraggedObjWindow((FrameworkElement)sender);
                draggedObjWindow.Show();

                DragDrop.DoDragDrop(this, dragData, DragDropEffects.Move);

                draggedObjWindow.Close();
                draggedObjWindow = null;
                this.QueryContinueDrag -= queryHandler;
                this.GiveFeedback -= handler;

                e.Handled = true;
            }
        }

        Window creatDraggedObjWindow(FrameworkElement e)
        {
            DragObjectWindow w = new DragObjectWindow();
            w.WindowStyle = WindowStyle.None;
            w.AllowsTransparency = true;
            w.AllowDrop = false;
            w.Background = null;
            w.IsHitTestVisible = false;
            w.SizeToContent = SizeToContent.WidthAndHeight;
            w.Topmost = true;
            w.ShowInTaskbar = false;

            w.Content = new SimpleNodePresenter((SimpleNode)e.Tag,gp);

            updatePos(w);

            return w;
        }

        struct Pt
        {
            public int X;
            public int Y;
        }

        [System.Runtime.InteropServices.DllImport("user32.dll")]
        static extern bool GetCursorPos(out Pt pt);

        void updatePos(Window w)
        {
            Pt pt = new Pt();
            if (GetCursorPos(out pt))
            {
                w.Left = (double)pt.X;
                w.Top = (double)pt.Y;
            }
        }

        void Drag_Query(object sender, QueryContinueDragEventArgs e)
        {
            if (draggedObjWindow != null)
            {
                updatePos(draggedObjWindow);
            }
        }

        void Drag_GiveFeedback(object sender, GiveFeedbackEventArgs e)
        {
            System.Windows.Input.Mouse.SetCursor(System.Windows.Input.Cursors.Arrow);

            e.UseDefaultCursors = false;
            e.Handled = true;
        }

        void RowColumn_Drop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent("GroupNodeChild"))
            {
                FrameworkElement TextBlockElement = sender as FrameworkElement;
                SimpleNode dropNode = e.Data.GetData("GroupNodeChild") as SimpleNode;
                SimpleNode thisNode = TextBlockElement.Tag as SimpleNode;

                if (Node.Nodes.Contains(dropNode))
                {
                    if (dropNode != thisNode)
                    {
                        //Umsortieren
                        int index = this.Node.Nodes.IndexOf(thisNode);

                        //Feststellen ob Mouse oberhalb oder unterhalb der Mitte war:
                        Point mousePoint = e.GetPosition(TextBlockElement);
                        bool insertBefore = true;

                        if (TextBlockElement.ActualWidth > TextBlockElement.ActualHeight)
                        {
                            if (TextBlockElement.ActualHeight / 2 < mousePoint.Y)
                                insertBefore = false;
                        }
                        else
                        {
                            if (TextBlockElement.ActualWidth / 2 < mousePoint.X)
                                insertBefore = false;
                        }

                        if (!insertBefore)
                            index++;
                        if (index > Node.Nodes.IndexOf(dropNode))
                            index--;

                        Node.Nodes.Remove(dropNode);
                        Node.Nodes.Insert(index, dropNode);

                        e.Handled = true;
                    }
                }
            }
        }

        void GroupNodePresenter_PreviewMouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if ((e.OriginalSource is TextBlock) == false)
            {
                if (e.LeftButton == System.Windows.Input.MouseButtonState.Pressed && !Keyboard.IsKeyDown(Key.LeftCtrl))
                {
                    Point mPos = e.GetPosition(this);
                    dragPoint = new Point(mPos.X - (this.DesiredSize.Width / 2), mPos.Y - (this.DesiredSize.Height / 2));
                    DataObject dragData = new DataObject("GroupNode", this);
                    gp.BringToBottom(this);
                    DragDrop.DoDragDrop(this, dragData, DragDropEffects.Move);
                }
            }
        }

        void GroupNodePresenter_Drop(object sender, DragEventArgs e)
        {
            if (e.Data.GetData("SimpleNode") is SimpleNodePresenter)
            {
                GroupNode target = this.Node;

                SimpleNode dObject = (e.Data.GetData("SimpleNode") as SimpleNodePresenter).Node;
                target.AddNode(dObject);
                gp.Nodes.Remove(dObject);

                e.Handled = true;
            }
            if (e.Data.GetData("GroupNode") is GroupNodePresenter)
            {
                if ((e.Data.GetData("GroupNode") as GroupNodePresenter) == this)
                    return;

                GroupNode target = this.Node;

                GroupNode dObject = (e.Data.GetData("GroupNode") as GroupNodePresenter).Node;

                foreach (SimpleNode s in dObject.Nodes)
                {
                    target.AddNode(s);
                }

                gp.Nodes.Remove(dObject);

                e.Handled = true;
            }
        }

        void CreateMatrix()
        {
            foreach (UIElement uie in Matrix.Children)
            {
                uie.PreviewMouseMove -= RowColumn_PreviewMouseMove;
            }

            //Grid zusammenstellen
            Matrix.RowDefinitions.Clear();
            Matrix.ColumnDefinitions.Clear();
            Matrix.Children.Clear();
            horizontalBlocks.Clear();
            verticalBlocks.Clear();

            for (int i = 0; i <= Node.Nodes.Count + 1; i++)
            {
                RowDefinition rd2 = new RowDefinition();
                rd2.Height = GridLength.Auto;
                Matrix.RowDefinitions.Add(rd2);
                ColumnDefinition cd2 = new ColumnDefinition();
                cd2.Width = GridLength.Auto;
                Matrix.ColumnDefinitions.Add(cd2);
            }

            //Namen reinschreiben
            for (int i = 0; i < Node.Nodes.Count; i++)
            {
                TextBlock rowBlock = new TextBlock();
                rowBlock.Tag = Node.Nodes[i];
                rowBlock.IsHitTestVisible = gp.ShowLabels;
                rowBlock.PreviewMouseMove += new System.Windows.Input.MouseEventHandler(RowColumn_PreviewMouseMove);
                rowBlock.Drop += new DragEventHandler(RowColumn_Drop);
                if (gp.ShowLabels)
                    rowBlock.Text = Node.Nodes[i].Name;
                rowBlock.SnapsToDevicePixels = true;
                rowBlock.Padding = new Thickness(5.0, 1.0, 5.0, 1.0);
                Grid.SetColumn(rowBlock, 0);
                Grid.SetRow(rowBlock, i + 1);
                Matrix.Children.Add(rowBlock);
                horizontalBlocks.Add(rowBlock);

                TextBlock rowBlock2 = new TextBlock();
                rowBlock2.Tag = Node.Nodes[i];
                rowBlock2.IsHitTestVisible = gp.ShowLabels;
                rowBlock2.PreviewMouseMove += new System.Windows.Input.MouseEventHandler(RowColumn_PreviewMouseMove);
                rowBlock2.Drop += new DragEventHandler(RowColumn_Drop);
                if (gp.ShowLabels)
                    rowBlock2.Text = Node.Nodes[i].Name;
                rowBlock2.SnapsToDevicePixels = true;
                rowBlock2.Padding = new Thickness(5.0, 1.0, 5.0, 1.0);
                Grid.SetColumn(rowBlock2, Node.Nodes.Count + 1);
                Grid.SetRow(rowBlock2, i + 1);
                Matrix.Children.Add(rowBlock2);

                TextBlock columnBlock = new TextBlock();
                columnBlock.Tag = Node.Nodes[i];
                columnBlock.IsHitTestVisible = gp.ShowLabels;
                columnBlock.PreviewMouseMove += new System.Windows.Input.MouseEventHandler(RowColumn_PreviewMouseMove);
                columnBlock.Drop += new DragEventHandler(RowColumn_Drop);
                columnBlock.SnapsToDevicePixels = true;
                if (gp.ShowLabels)
                    columnBlock.Text = Node.Nodes[i].Name;
                columnBlock.LayoutTransform = new RotateTransform(-90);
                columnBlock.Padding = new Thickness(5.0, 1.0, 5.0, 1.0);
                Grid.SetColumn(columnBlock, i + 1);
                Grid.SetRow(columnBlock, 0);
                Matrix.Children.Add(columnBlock);
                verticalBlocks.Add(columnBlock);

                TextBlock columnBlock2 = new TextBlock();
                columnBlock2.Tag = Node.Nodes[i];
                columnBlock2.IsHitTestVisible = gp.ShowLabels;
                columnBlock2.PreviewMouseMove += new System.Windows.Input.MouseEventHandler(RowColumn_PreviewMouseMove);
                columnBlock2.Drop += new DragEventHandler(RowColumn_Drop);
                columnBlock2.SnapsToDevicePixels = true;
                if (gp.ShowLabels)
                    columnBlock2.Text = Node.Nodes[i].Name;
                columnBlock2.LayoutTransform = new RotateTransform(-90);
                columnBlock2.Padding = new Thickness(5.0, 1.0, 5.0, 1.0);
                Grid.SetColumn(columnBlock2, i + 1);
                Grid.SetRow(columnBlock2, Node.Nodes.Count + 1);
                Matrix.Children.Add(columnBlock2);
            }

            foreach (VizEdge edge in Node.InnerConnections)
            {
                if (Node.Nodes.Contains(edge.Head) && Node.Nodes.Contains(edge.Tail))
                {
                    int indexHead = Node.Nodes.IndexOf(edge.Head as SimpleNode);
                    int indexTail = Node.Nodes.IndexOf(edge.Tail as SimpleNode);
                    CreateMark(indexHead, indexTail);
                    CreateMark(indexTail, indexHead);
                }
            }

            Matrix.Measure(new Size(Double.PositiveInfinity, Double.PositiveInfinity));
            Matrix.Arrange(new Rect(Matrix.DesiredSize));

            double titleHeight = 0;

            if (gp.ShowTitles)
                titleHeight += 20;

            Node.Size = new Size(Matrix.DesiredSize.Width + 14, Matrix.DesiredSize.Height + 14 + titleHeight);
            

            //Position im GroupNode setzen
            Node.NodesPosition.Clear();
            for (int i = 0; i < verticalBlocks.Count; i++)
            {
                Point relHorizontalPoint = horizontalBlocks[i].TranslatePoint(new Point(0, horizontalBlocks[i].ActualHeight / 2.0), this.Matrix);
                Node.NodesPosition.Add(new Point(relHorizontalPoint.Y + 7, relHorizontalPoint.Y + 7 + titleHeight));
            }

            OnPropertyChanged("Matrix");
            this.Node.OnPropertyChanged("Pos");
        }

        private void CreateMark(int _rowIndex, int _columnIndex)
        {
            Border mark = new Border();
            mark.BorderThickness = new Thickness(0);
            mark.SnapsToDevicePixels = true;
            mark.Margin = new Thickness(0);
            mark.BorderBrush = Brushes.Black;
            mark.Background = Brushes.Aquamarine;
            mark.VerticalAlignment = VerticalAlignment.Stretch;
            mark.HorizontalAlignment = HorizontalAlignment.Stretch;

            Grid.SetColumn(mark, _columnIndex + 1);
            Grid.SetRow(mark, _rowIndex + 1);
            Matrix.Children.Add(mark);
        }

        void Nodes_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            CreateMatrix();
        }



        public GroupNode Node
        {
            get { return (GroupNode)GetValue(NodeProperty); }
            set { SetValue(NodeProperty, value); }
        }

        public override BaseNode Data
        {
            get { return Node; }
        }

        public static readonly DependencyProperty NodeProperty = 
            DependencyProperty.Register("Node", typeof(GroupNode), typeof(GroupNodePresenter));

        public Grid Matrix
        {
            get { return (Grid)GetValue(MatrixProperty); }
            set { SetValue(MatrixProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Matrix.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty MatrixProperty =
            DependencyProperty.Register("Matrix", typeof(Grid), typeof(GroupNodePresenter));

        void Node_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName.Equals("Pos"))
            {
                //gp.UpdateVisuals();
                //gp.InvalidateVisual();
                gp.InvalidateMeasure();
                //gp.InvalidateArrange();
            }
            if (e.PropertyName == "Title")
                OnPropertyChanged("Title");
        }
    }
}
