﻿// --------------------------------------------------------------------------------------------------------------------
// <info file="CsharpInputBox.cs" author="Rouven Himmelstein" version="1.2.0.0" releaseDate="21.05.2012">
//   (c) by Rouven Himmelstein
// </info>
// --------------------------------------------------------------------------------------------------------------------
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace Vis2NodeTrix
{
    /// <summary>
    /// Shows a InputBox dialog.
    /// </summary>
    static class InputBox
    {
        static Window dialogWindow;
        static InputBoxResult PressedButton;

        private enum InputBoxResult
        {
            Ok = 0,
            Cancel = 1,
            Default = 2,
        }

        /// <summary>
        /// Shows the InputBox.
        /// </summary>
        /// <param name="title">Title of the InputBox Dialog window.</param>
        /// <param name="description">Description of the value being entered.</param>
        /// <param name="defaultInputValue">Default value of the InputBox.</param>
        /// <returns></returns>
        public static string ShowDialog(string title, string description, string defaultInputValue)
        {
            var rootPanel = new StackPanel { Margin = new Thickness(5) };
            var AA = new StackPanel { Orientation = Orientation.Horizontal };
            var AB = new StackPanel { };

            rootPanel.Children.Add(AA);
            rootPanel.Children.Add(AB);

            var AAA = new StackPanel { Width = 215, Margin = new Thickness(2.5) };
            var AAB = new StackPanel { Width = 75, Orientation = Orientation.Vertical };

            AA.Children.Add(AAA);
            AA.Children.Add(AAB);

            dialogWindow = new Window
            {
                Title = title,
                WindowStartupLocation = WindowStartupLocation.CenterOwner,
                WindowStyle = WindowStyle.ThreeDBorderWindow,
                ResizeMode = ResizeMode.NoResize,
                Width = 315,
                Height = 120,
                Content = rootPanel,
                ShowInTaskbar = false,
                Background = Brushes.LightGray
            };

            var descriptionText = new TextBlock { Text = description, TextWrapping = TextWrapping.Wrap, MaxHeight = 50 };
            var inputTextBox = new TextBox { Text = defaultInputValue };
            var OkButton = new Button { Content = "Ok", Margin = new Thickness(0, 0, 0, 5) };
            var CancelButton = new Button { Content = "Abbrechen", Margin = new Thickness(0, 0, 0, 5) };

            OkButton.Click += new RoutedEventHandler(OkButton_Click);
            CancelButton.Click += new RoutedEventHandler(CancelButton_Click);
            dialogWindow.KeyDown += new System.Windows.Input.KeyEventHandler(dialogWindow_KeyDown);

            AAA.Children.Add(descriptionText);
            AB.Children.Add(inputTextBox);
            AAB.Children.Add(OkButton);
            AAB.Children.Add(CancelButton);

            PressedButton = InputBoxResult.Default;

            inputTextBox.Focus();
            inputTextBox.SelectAll();

            dialogWindow.ShowDialog();

            switch (PressedButton)
            {
                case InputBoxResult.Ok:
                    return inputTextBox.Text;
                case InputBoxResult.Cancel:
                    return string.Empty;
                default:
                    return string.Empty;
            }
        }
        private static void OkButton_Click(object sender, RoutedEventArgs e)
        {
            PressedButton = InputBoxResult.Ok;
            dialogWindow.Close();
        }

        private static void CancelButton_Click(object sender, RoutedEventArgs e)
        {
            PressedButton = InputBoxResult.Cancel;
            dialogWindow.Close();
        }

        private static void dialogWindow_KeyDown(object sender, System.Windows.Input.KeyEventArgs e)
        {
            switch (e.Key)
            {
                case System.Windows.Input.Key.Enter:
                    PressedButton = InputBoxResult.Ok;
                    dialogWindow.Close();
                    break;
                case System.Windows.Input.Key.Escape:
                    PressedButton = InputBoxResult.Cancel;
                    dialogWindow.Close();
                    break;
            }
        }
    }
}