﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GraphVizNet;
using System.Windows;
using System.Windows.Media;

namespace Vis2NodeTrix
{
    /// <summary>
    /// Represents a simple node
    /// </summary>
    public class SimpleNode : BaseNode
    {
        private List<VizEdge> connections;
        private GroupNode parentNode;

        /// <summary>
        /// Returns the parent node of the node. Returns null if the node is not grouped
        /// </summary>
        public GroupNode ParentNode
        {
            get
            {
                return parentNode;
            }
            set
            {
                OnPropertyChanged("BeforeParentNode");

                parentNode = value;

                OnPropertyChanged("ParentNode");
            }
        }

        /// <summary>
        /// Returns a value indicating if the node is in a groupnode
        /// </summary>
        public bool IsGrouped { get { return !(ParentNode == null); } }

        /// <summary>
        /// List of connections
        /// </summary>
        public override List<VizEdge> Connections
        {
            get { return connections; }
        }

        /// <summary>
        /// Initializes a new instance of the SimpleNode class
        /// </summary>
        public SimpleNode()
        {
            connections = new List<VizEdge>();

            this.Size = new System.Windows.Size(80, 30);
            IsAnimating = false;
        }

        /// <summary>
        /// Adds a new connection to this node
        /// </summary>
        /// <param name="_node">The other node</param>
        /// <param name="_edge">The edge connection the nodes</param>
        /// <param name="_noCall">If true the other nodes AddOriginalNode method is not called</param>
        public void AddOriginalNode(SimpleNode _node, VizEdge _edge = null, bool _noCall = false)
        {
            VizEdge edge = _edge;

            //Wenn _edge, diese verwenden
            if (edge == null)
            {
                edge = new VizEdge();
                edge.Head = this;
                edge.Tail = _node;
            }

            connections.Add(edge);

            if (!_noCall)
                _node.AddOriginalNode(this, edge, true);
        }

        /// <summary>
        /// Returns the next valid point on the nodes boundingbox in direction of the edge
        /// </summary>
        /// <param name="_edge">Edge defining the direction</param>
        /// <returns>The nearest point on the boundingbox</returns>
        public override Point PointOnBox(VizEdge _edge)
        {
            Vector direction;

            BaseNode headNode = _edge.Head as BaseNode;
            if ((_edge.Head as SimpleNode).IsGrouped)
                headNode = (_edge.Head as SimpleNode).ParentNode;

            BaseNode tailNode = _edge.Tail as BaseNode;
            if ((_edge.Tail as SimpleNode).IsGrouped)
                tailNode = (_edge.Tail as SimpleNode).ParentNode;


            if (tailNode == this)
            {
                Vector d1 = (tailNode.Pos - _edge.Points[_edge.Points.Count - 1]);
                Vector d2 = (tailNode.Pos - headNode.Pos);
                d1.Normalize();
                d2.Normalize();

                if (Double.IsNaN(d1.X))
                    d1.X = 0.0;
                if (Double.IsNaN(d1.Y))
                    d1.Y = 0.0;

                direction = d1 *4.0 + d2;
            }
            else
            {
                Vector d1 = (headNode.Pos - _edge.Points[1]);
                Vector d2 = (headNode.Pos - tailNode.Pos);
                d1.Normalize();
                d2.Normalize();

                if (Double.IsNaN(d1.X))
                    d1.X = 0.0;
                if (Double.IsNaN(d1.Y))
                    d1.Y = 0.0;

                direction = d1 *4.0 + d2;
            }

            direction.Normalize();


            double xDistance = Double.PositiveInfinity;
            double yDistance = Double.PositiveInfinity;

            if (direction.Y != 0.0)
                yDistance = (Size.Height / 2.0) / direction.Y;
            if (direction.X != 0.0)
                xDistance = (Size.Width / 2.0) / direction.X;

            if (Math.Abs(xDistance) < Math.Abs(yDistance))
            {
                //In X-Richtung
                return this.Pos - direction * (Math.Abs(xDistance) + 5.0);
            }
            else
            {
                return this.Pos - direction * (Math.Abs(yDistance) + 5.0);
            }
        }
    }
}
