﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GraphVizNet;
using System.Windows.Controls;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Input;

namespace Vis2NodeTrix
{
    /// <summary>
    /// Gives an visual abstraction of the SimpleNodePresenter
    /// </summary>
    public class SimpleNodePresenter : BaseNodePresenter
    {
        /// <summary>
        /// The point where the dragging had started
        /// </summary>
        public Point dragPoint;

        static SimpleNodePresenter()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(SimpleNodePresenter),
                new FrameworkPropertyMetadata(typeof(SimpleNodePresenter)));
        }

        /// <summary>
        /// Initializes a new instance of the SimpleNodePresenter class
        /// </summary>
        /// <param name="_node">The underlaying node</param>
        /// <param name="_gp">The graphpresenter in which the node is shown</param>
        public SimpleNodePresenter(SimpleNode _node, GraphPresenter _gp) : base(_gp)
        {
            Node = _node;
            Node.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(Node_PropertyChanged);
            
            this.SnapsToDevicePixels = true;

            this.PreviewMouseMove += new System.Windows.Input.MouseEventHandler(SimpleNodePresenter_PreviewMouseMove);
            this.DragOver += new DragEventHandler(SimpleNodePresenter_DragOver);
            this.Drop += new DragEventHandler(SimpleNodePresenter_Drop);

            initialiseHandler();
        }

        void SimpleNodePresenter_PreviewMouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (e.LeftButton == System.Windows.Input.MouseButtonState.Pressed && !Keyboard.IsKeyDown(Key.LeftCtrl))
            {
                Point mPos = e.GetPosition(this);
                dragPoint = new Point(mPos.X - (this.DesiredSize.Width / 2), mPos.Y - (this.DesiredSize.Height / 2));
                DataObject dragData = new DataObject("SimpleNode", this);
                gp.BringToBottom(this);
                DragDrop.DoDragDrop(this, dragData, DragDropEffects.Move);
            }

            e.Handled = true;
        }

        void SimpleNodePresenter_Drop(object sender, DragEventArgs e)
        {
            if (e.Data.GetData("SimpleNode") is SimpleNodePresenter)
            {
                if ((e.Data.GetData("SimpleNode") as SimpleNodePresenter).Node == this.Node)
                {
                    e.Handled = false;
                    return;
                }

                SimpleNode target = this.Node;

                SimpleNode dObject = (e.Data.GetData("SimpleNode") as SimpleNodePresenter).Node;
                GroupNode group = new GroupNode();
                group.Pos = target.Pos;

                gp.Nodes.Remove(dObject);
                gp.Nodes.Remove(target);

                group.AddNode(dObject);
                group.AddNode(target);

                gp.Nodes.Add(group);

                e.Handled = true;
            }
            if (e.Data.GetData("GroupNodeChild") is SimpleNode)
            {
                SimpleNode target = this.Node;

                SimpleNode dObject = e.Data.GetData("GroupNodeChild") as SimpleNode;
                
                GroupNode oldGroup = dObject.ParentNode;
                
                oldGroup.Nodes.Remove(dObject);
                if (oldGroup.Nodes.Count == 0)
                {
                    gp.Nodes.Remove(oldGroup);
                }

                if (oldGroup.Nodes.Count == 1)
                {
                    SimpleNode fn = oldGroup.Nodes.First();
                    oldGroup.Nodes.Remove(fn);
                    fn.Pos = oldGroup.Pos;
                    gp.Nodes.Remove(oldGroup);
                    gp.Nodes.Add(fn);
                }

                GroupNode group = new GroupNode();
                group.AddNode(dObject);
                group.AddNode(target);

                gp.Nodes.Remove(dObject);
                gp.Nodes.Remove(target);

                group.Pos = target.Pos;

                gp.Nodes.Add(group);

                e.Handled = true;
            }
        }

        void SimpleNodePresenter_DragOver(object sender, DragEventArgs e)
        {
            if (!((e.Data.GetData("SimpleNode") is SimpleNodePresenter) || e.Data.GetData("GroupNodeChild") is SimpleNode))
            {
                e.Effects = DragDropEffects.None;
                e.Handled = false;
            }
        }

        #region Properties

        /// <summary>
        /// Returns the unerlaying node of this presenter
        /// </summary>
        public override BaseNode Data
        {
            get { return Node; }
        }

        /// <summary>
        /// Returns the unerlaying node of this presenter
        /// </summary>
        public SimpleNode Node
        {
            get { return (SimpleNode)GetValue(NodeProperty); }
            set { SetValue(NodeProperty, value); }
        }

        /// <summary>
        /// DependencyProperty for the Node Property
        /// </summary>
        public static readonly DependencyProperty NodeProperty =
            DependencyProperty.Register("Node", typeof(SimpleNode), typeof(SimpleNodePresenter));

        #endregion

        void Node_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if(e.PropertyName.Equals("Pos"))
            {
                //gp.UpdateVisuals();
                //gp.InvalidateVisual();
                gp.InvalidateMeasure();
                //gp.InvalidateArrange();
            }
        }
    }
}
